/*
  file:         Basis_table.cc
  purpose:      routines dealing with Basis Table
  created:      David P Jacobs May 1990
  modified:     pasha jun 18-19 2019
  modification: - Deg_of_last_basis, Deg_to_basis_table turned to class members
                - many funcs turned to Albert methods
				- some functions eliminated (called once)
*/

#include <stdlib.h>
#include <iostream>
#include "albert.hh"

#include "Basis_table.hh"

#ifdef DMALLOC
#include <dmalloc.h>
#endif

//bool InitializeDegtoBasisTable (int);
void UpdateDegToBasisTable (int, Basis);


// Enter a New Basis into Basis Table.
// GLOBALS MODIFIED:                                 
//     Basis_table -- New Basis is entered.          
//     Nextbasistobefilled -- Incremented.           
// args: Left_factor -- of the new Basis.                    
//       Right_factor -- of the new Basis.                     
//       Type -- of the new Basis.                               
// RETURNS: Basis Number of the new Basis entered.                  
Basis Albert::EnterBasis 
(const Basis Left_factor, const Basis Right_factor, const Name cur_type)
{
  Basis bn = Basis_table.size();
  BT_rec br;
  br.left_factor  = Left_factor;
  br.right_factor = Right_factor;
  br.type = cur_type;
  Basis_table.push_back (br);
  UpdateDegToBasisTable (GetDegreeName (cur_type), bn);
  return (bn);
}



void Albert::UpdateDegToBasisTable (const int Deg, const Basis Cur_basis)
{
  if (Deg > this->Deg_of_last_basis) 
	{
	  this->Deg_to_basis_table[Deg-1].first_basis = Cur_basis;
	  this->Deg_to_basis_table[Deg-1].last_basis  = Cur_basis;
	  this->Deg_of_last_basis = Deg;
    }
  else
	{
	  (this->Deg_to_basis_table[Deg-1].last_basis)++;
	}
}


Basis Albert::BasisStart (const Degree Deg) const
{
  return (this->Deg_to_basis_table[Deg-1].first_basis);
}


Basis Albert::BasisEnd (const Degree Deg) const
{
  return (this->Deg_to_basis_table[Deg-1].last_basis);
}


// RETURNS: Type of the Basis element B
Name Albert::GetType (const Basis B) const
{
  return (this->Basis_table[B].type);
}

// end of Basis_table.cc
