/*
  file:         CreateSubs.cc
  created:      David P Jacobs circa 1990
  modified:     Pasha Zusmanovich apr 5 2014,feb 13,18,20-22,may 18 2016
  modification: almost everything transferred to albert.hh
*/

// Given a set partitioning, we create substitution records by selecting basis 
// elements of the type given in set partition.
//
// e.g.                                                    
//
//       If Set partitions are :                         
//                                                       
//                              x       y                
//                           ---------------             
//                        u | 1 0 0 | 1 0 0 |            
//                           ---------------             
//                        v | 0 2 0 |                    
//                           -------                     
//                        w | 0 0 1 |                    
//                           -------                     
//                                                       
//            and the generators are  a, b and c   then the
//     substitution record would be as follows:            
//                                                         
//                              x       y                  
//                           ---------------               
//                        u |  ab   |   a   |              
//                           ---------------               
//                        v |   b   |                      
//                           -------                       
//                        w |   c   |                      
//                           -------                       
//                                                         
//     So the identity is linearized F(x,y) --> F(u,v,w,y).
//     We substiute basis elements in place of variables u,v,w,y.
//


#include <iostream>
#include "albert.hh"

#ifdef DMALLOC
#include <dmalloc.h>
#endif

#if DEBUG_SUBSTITUTION
void Albert::PrintSubstitution() const
{
  static int count = 1;
  std::cout << "Substitution " << count++ << " is " << std::endl;
  int j;
  for (int i=0; i < this->Num_vars;i++) 
	{
	  for (j=0; j < this->Deg_var[i]; j++)
		{
		  std::cout << this->Substitution[i * (this->Max_deg_var) + j];
		}
	  std::cout << std::endl;
    }
}
#endif


// end of CreateSubs.cc
