/*
  file:         GF2.hh
  purpose:      implements GF(2)
  created:      pasha mar 2,8-9 2016
  modified:     pasha jul 27-28 2019
  modification: some operators moved to upper class
*/

#include "scalar.hh"


class GF2_element: public Scalar<bool>
{
public:

  inline GF2_element () {}

  inline GF2_element (const long long i)
  {
	this->val = (bool)i;
  }

  friend inline GF2_element operator- (const GF2_element& X)
  {
	GF2_element M = X;
	return (M);
  }

  friend inline GF2_element operator+ (const GF2_element& X, 
									   const GF2_element& Y)
  {
	GF2_element S;
	S.val = ((X.val == Y.val) ? 0 : 1);
	return (S);
  }


  friend inline GF2_element operator* (const GF2_element& X, 
									   const GF2_element& Y)
  {
	GF2_element P;
	P.val = (((X.val == 0) || (Y.val == 0)) ? 0 : 1);
	return (P);
  }

  
  // 1/X
  friend inline GF2_element S_inv (const GF2_element& X)
  {
	if (X.val == 0)
	  {
		std::cerr << "division by zero in GF(2)" << std::endl;
		exit (-1);
	  }
	return ((GF2_element)1);
  }
  

  inline GF2_element& operator*= (const GF2_element& X)
  {
	*this = *this * X;
	return (*this);
  }

}; // end of class GF2_element



// end of GF2.hh
