/*
  file:         GenerateEquations.h
  created:      Pasha Zusmanovich feb 21 2009
  modified:     pasha feb 18,20,23-24,mar 9,may 18 2016
  modification: - c++ file ext
                - templated Scalar
                - include once
				- restored debug functions
*/

#ifndef __GENERATE_EQUATIONS_H
#define __GENERATE_EQUATIONS_H

#include "assert_not_null.hh"
#include "Debug.hh"
#include "Memory_routines.hh"
#include "Po_parse_exptext.hh"


template<typename T> bool FreeEqns (Eqn_list_node<T>* L)
{
  assert_not_null(L);
  if (L->next == NULL) 
	{
	  if (L->basis_pairs != NULL)
		{
		  free (L->basis_pairs);
		}
	  free (L);
	}
  else 
	{
	  FreeEqns<T> (L->next);
	  free (L->basis_pairs);
	  free (L);
	}
  return (1);
}




template<typename T> Eqn_list_node<T>* GetNewEqnListNode()
{
  Eqn_list_node<T>* temp_node = 
	(Eqn_list_node<T>*)(Mymalloc (sizeof(Eqn_list_node<T>)));
  assert_not_null(temp_node);
  temp_node->basis_pairs = NULL;
  temp_node->next = NULL;
  return (temp_node);
}



#if DEBUG_EQNS
template<typename T> bool PrintEqn (const Basis_pair<T>* Temp_eqn)
{
  assert_not_null(Temp_eqn);
  int i=0;
  int j=0;
  while (Temp_eqn[i].coef != (T)0) 
	{
	  std::cout << " + ";
	  std::cout << Temp_eqn[i].coef << " b[" 
				<< Temp_eqn[i].left_basis << "] b[" 
				<< Temp_eqn[i].right_basis << "d]";
	  i++;
	  j = (j+1)%4;
	  if (j == 0)
		{
		  std::cout << std::endl;
		}
    }
  j = 0;
  std::cout << std::endl;
  return (1);
}
#endif // DEBUG_EQNS


#endif // __GENERATE_EQUATIONS_H
// end of GenerateEquations.hh
