/*
  file:         Id_routines.cc
  purpose:      This module contains routines to enter, remove or print 
                identities in the Id_queue.
  created:      Sekhar Muddana
  modified:     Pasha Zusmanovich feb 13,18,21 2016
  modification: support for dmalloc; c++ file ext.; cleanup
*/

#include <stdlib.h>
#include <string.h>
#include <iostream>
#include "Memory_routines.hh"
#include "Po_parse_exptext.hh"
#include "Po_routines.hh"

#include "Id_routines.hh"

#ifdef DMALLOC
#include <dmalloc.h>
#endif



// Allocate space for an id_queue_node.
// Change its fields to the fields supplied. (Str,Id)
// Attach this node to the end of the Id_queue.
// This routine is called when the command "i identity" is issued.
// MODIFIES: Id_queue -- pointer the head of the queue of identities.
// args: Id -- pointer to the identity to be added to the queue.
//       Str -- identity as a string entered by user. "i [identity]"
// RETURNS: number assigned to the new identity
int Add_id 
(struct polynomial* Id, const char Str[], struct id_queue_head* Id_queue)
{
  struct id_queue_node* pntr = Id_queue_node_alloc();
  pntr->identity = Id;
  pntr->user_str = (char*)Mymalloc (strlen (Str)+1);
  strcpy (pntr->user_str, Str);
  pntr->next = NULL;

  if (Id_queue->first == NULL) 
	{
	  Id_queue->first = pntr;
	  return (1);
    }
  else 
	{
	  struct id_queue_node* temp = Id_queue->first;
	  int id_no = 1;
	  id_no++;
	  while (temp->next != NULL) 
		{
		  temp = temp->next;
		  id_no++;
		}
	  temp->next = pntr;
	  return (id_no);
	}
} // end of Add_id()




// skip (Id_no - 1) number of elements and delete the Id_no element from the 
// queue.
// This routine is called when the command "r number" is issued.
// MODIFIES: Id_queue -- pointer the head of the queue of identities.
// arg: Id_no -- the number of the identity to be removed from the queue.
// RETURNS: 1 if the identity with the given Id_no is removed, 0 otherwise
int Remove_id (int Id_no, struct id_queue_head *Id_queue)
{
  struct id_queue_node* pntr1 = Id_queue->first;
  struct id_queue_node* pntr2;
  if (pntr1 != NULL)
	{
	  pntr2 = pntr1->next;
	}
  else
	{
	  // No elements in the queue!
	  return (0);
	}
  if (Id_no == 1) 
	{
	  Id_queue->first = pntr1->next;
	  free (pntr1->user_str);
	  DestroyPoly (pntr1->identity);
	  free (pntr1);
	  return (1);
    }
  else 
	{
	  while ((--Id_no > 1) && (pntr2 != NULL)) 
		{
		  pntr1 = pntr2;
		  pntr2 = pntr2->next; 
        }
	  if (pntr2 == NULL)
		{
		  return(0);
		}
	  else 
		{
		  pntr1->next = pntr2->next;
		  free (pntr2->user_str);
		  DestroyPoly (pntr2->identity);
		  free (pntr2);
		  return(1);
        }
    }
} // end of Remove_id()



// Delete all elements in the Id_queue.
// This routine is called when the command "r *" is issued.
// MODIFIES: Id_queue -- pointer the head of the queue of identities
void Remove_all_ids (struct id_queue_head* Id_queue)
{
  while (Remove_id (1, Id_queue)) {};
}



// Print the identity number and it's string, of all elements in the Id_queue.
// This routine is called when the command "d" is issued.
// REQUIRES: Id_queue -- pointer the head of the queue of identities
void Print_ids (struct id_queue_head Id_queue)
{
  struct id_queue_node* pntr = Id_queue.first;
  unsigned int i = 1;
  while (pntr != NULL) 
	{
	  std::cout << "  " << (i++) << ". " << pntr->user_str << " " << std::endl;
	  pntr = pntr->next;
    }
}


// end of Id_routines.cc
