/*
  file:         Mult_table_print.hh
  created:      Pasha Zusmanovich mar 9 2009
  modified:     pasha jun 26,29-30 2019
  modification: minor code streamlining
*/

#ifndef __MULT_TABLE_PRINT_H
#define __MULT_TABLE_PRINT_H

#include <iomanip>
#include <sstream>
#include "Alg_elements.hh"

#if __MINGW32__
#define fprintf __mingw_fprintf
#endif


template<typename T> void Print_AE (const typename Alg_element<T>::type& ae, 
									std::ostream& filePtr, 
									const bool output_for_gap)
{
  unsigned int trmcnt = 0;	   // How many terms have been printed
  unsigned int lnecnt = 0;	   // How many have been printed on the current line
  
  typename std::map<Basis, T>::const_iterator aei;
  std::stringstream ss;
  for (aei = ae.begin(); aei != ae.end(); aei++)
	{
	  if ((aei->first != 0) && (aei->second != (T)0))
		{
		  if (output_for_gap)
			{
			  ss << aei->second << ", " << aei->first << ", ";
			}
		  else
			{
			  if ((trmcnt > 0) && (trmcnt % 4 == 0)) 
				{
				  // 4 items per line
				  filePtr << std::endl;
				  lnecnt = 0;
				}
			  filePtr << ((lnecnt == 0) ? " " : " +") << "  " << std::right
					  << std::setw(3) << aei->second << " b" << std::left 
					  << std::setw(3) << aei->first;
			  trmcnt++;
			  lnecnt++;
			}
		}
	}
  if (output_for_gap)
	{
	  // remove the last ", "
	  std::string s = ss.str();
	  s.resize (s.size() - 2);
	  filePtr << s;
	}
}


#endif // __MULT_TABLE_PRINT_H
// end of Mult_table_print.hh
