/*
  file:         Po_parse_exptext.cc
  purpose:      routines to parse the expanded string
  created:      Jeff Offutt circa 1990
  modified:     Pasha Zusmanovich feb 11,18,mar 2 2016
  modification: - added dmalloc
                - minor cleanup and code formatting
				- some int -> size_t
				- cleanup
				- c++ file ext.
*/

// The typical expanded string looks like this:     
//             -7x+10y+(x(yx))                  

#include <ctype.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "Memory_routines.hh"

#include "Po_parse_exptext.hh"

#ifdef DMALLOC
#include <dmalloc.h>
#endif

void CreateLR   (struct term_node*);
void Parse_term (struct term_node*, char[], size_t*);



// Scan the input string to create the polynomial, in its internal 
// representation.                                    
// The polynomial structure corresponding to Poly_str is created by calling 
// Parse_term() which creates the tree for the term string.
// MODIFIES: Poly_str -- Polynomial string to be converted to structure 
//           polynomial.                                    
// RETURNS: Pointer to created polynomial.                    
struct polynomial* Parse_exptext (char Poly_str[])
{
  char* term_str;                // string passed to Parse_term()
  int term_str_indx = 0;         // used to create string term_str
  size_t poly_str_indx = 0;      // position in the string being parsed
  size_t parse_str_indx = 0;     // hold the position from where to parse
  char sign = ' ';               // hold the sign of the scalar coef for term
  
  int i = 0, cur_coef = 0;	

  struct polynomial* poly = Poly_alloc();   // pointer that will be returned
  poly->degree = 0;
  for (i=0; i<NUM_LETTERS; i++) 
	{
	  poly->deg_letter[i]= 0; 
	}

  struct term_head* cur_head_ptr = Term_head_alloc();
  poly->terms = cur_head_ptr;
  cur_head_ptr->term = Term_node_alloc();

  if (Poly_str[poly_str_indx] == '-') 
	{
	  sign = '-';
	  poly_str_indx++;
    }

  if (! isdigit (Poly_str[poly_str_indx])) 
	{
	  if (sign != '-')
		{
		  cur_head_ptr->coef = 1;
		}
	  else
		{
		  cur_head_ptr->coef = -1;
		}
    }
  else 
	{
	  while (isdigit (Poly_str[poly_str_indx]))
		{
		  cur_coef = cur_coef * 10 + (Poly_str[poly_str_indx++] - '0');
		}
	  if (sign != '-')
		{
		  cur_head_ptr->coef = cur_coef;
		}
	  else
		{
		  cur_head_ptr->coef = 0 - cur_coef;
		}
	  sign = '+';
	  cur_coef = 0;
	}

  term_str = (char*)Mymalloc (strlen(Poly_str) + 1);
  while (poly_str_indx < strlen (Poly_str)) 
	{
	  if ((Poly_str[poly_str_indx] != '+') && (Poly_str[poly_str_indx] != '-'))
		{
		  term_str[term_str_indx++] = Poly_str[poly_str_indx++];
		}
	  else 
		{
		  sign = Poly_str[poly_str_indx++];
		  term_str[term_str_indx] = '\0';
		  term_str_indx = 0;
		  Parse_term (cur_head_ptr->term, term_str, &parse_str_indx);
		  parse_str_indx = 0;
		  cur_head_ptr->next = Term_head_alloc();
		  cur_head_ptr = cur_head_ptr->next;
		  cur_head_ptr->term = Term_node_alloc();
		  if (! isdigit (Poly_str[poly_str_indx])) 
			{
			  if (sign == '+')
				{
				  cur_head_ptr->coef = 1;
				}
			  else
				{
				  cur_head_ptr->coef = -1;
				}
			}
		  else 
			{
			  while (isdigit (Poly_str[poly_str_indx]))
				{
				  cur_coef = cur_coef * 10 + (Poly_str[poly_str_indx++] - '0');
				}
			  if (sign == '+')
				{
				  cur_head_ptr->coef = cur_coef;
				}
			  else
				{
				  cur_head_ptr->coef = 0 - cur_coef;
				}
			  cur_coef = 0;
			}
        }
	}
  term_str[term_str_indx] = '\0';
  Parse_term (cur_head_ptr->term, term_str, &parse_str_indx);
  free (term_str);
  return (poly);
} // end of Parse_exptext 



/* MODIFIES:                                                       */
/*     Root -- root of the tree for the term to be parsed.         */
/*     *Strptr -- Position where scanner is, in the input string.  */
/* REQUIRES:                                                       */
/*     String -- Term, for which to create a tree.                 */
/* FUNCTION:                                                       */
/*     Build the tree with the given root, by scanning the String. */
/* NOTE:                                                           */
/*     The tree is built recursively by bulding left and right sub */
/*     trees.                                                      */
void Parse_term (struct term_node* Root, char String[], size_t* Strptr)
{
  if (*Strptr < strlen(String)) 
    {
      if (String[*Strptr] == '(') 
		{
		  (*Strptr)++;
		  CreateLR (Root);
		  Parse_term (Root->left,  String, Strptr);
		  Parse_term (Root->right, String, Strptr);
		  (*Strptr)++;
		}
      else 
		{
		  Root->letter = String[*Strptr];
		  (*Strptr)++;
		}
    }
}



/* MODIFIES:                                                       */
/*     Root -- root of the tree for the term to be parsed.         */
/* FUNCTION:                                                       */
/*     Allocate space for the left child and the right child.      */ 
void CreateLR (struct term_node* Root)
{
  Root->left  = Term_node_alloc();
  Root->right = Term_node_alloc();
}


// end of Po_parse_exptext.cc
