/*
  file:         Po_semantics.cc
  purpose:      routines for doing Semantics while parsing the polynomial string
                These routines are called from Po_parse_poly.c
  created:      Sekhar Muddana circa 1990
  modified:     Pasha Zusmanovich dec 21 2013,feb 18,mar 8 2016
  modification: unused attribute at one place; cleanup; c++ file ext.
*/

#include <iostream>
#include "Memory_routines.hh"
#include "Po_parse_poly.hh"
#include "Po_syn_stack.hh"

#include "Po_semantics.hh"

struct unexp_tnode* Pop_tnode_ptr (struct unexp_tnode*[], int*);
void Push_tnode_ptr (struct unexp_tnode*[], int*, struct unexp_tnode*);



// 1.Semantic stack manipulation:
//        Manipulates the semantic stack by pushing and/or popping tnodes, 
//        depending on Reduction number.
// 2.Generation of partial tree.
// NOTE: There is a 1-1 correspondence between syntax and semantic stack
// MODIFIES:                                                    
//     Sem_stack -- Semantic stack, stack of pointers to tnodes.
//     *Sem_sp_ptr -- Semantic stack pointer.
// arg: Reduction_num -- Reduction number
void Reduce_semantics (int Reduction_num, struct unexp_tnode *Sem_stack[],
					   int* Sem_sp_ptr)
{
  struct unexp_tnode* Temp_tnode_ptr;	
  struct unexp_tnode* temp_tnode_ptr1;	
  struct unexp_tnode* temp_tnode_ptr2;
  // to cope with gcc >= 4.6(?) bogosity
  struct unexp_tnode* trash __attribute__ ((unused));

  switch (Reduction_num) 
	{
	case   1  : // polynomial -> term
	  break;

	case  2   : // polynomial -> - term
	  Temp_tnode_ptr = Unexp_tnode_alloc(); 
	  Temp_tnode_ptr->_operator = UNARY_MINUS;
	  Temp_tnode_ptr->operand1 = Pop_tnode_ptr(Sem_stack,Sem_sp_ptr); 
	  trash = Pop_tnode_ptr(Sem_stack,Sem_sp_ptr);
	  Push_tnode_ptr(Sem_stack,Sem_sp_ptr,Temp_tnode_ptr);
	  break;

	case  3   : // polynomial -> + term
	  Temp_tnode_ptr = Unexp_tnode_alloc(); 
	  Temp_tnode_ptr->_operator = UNARY_PLUS;
	  Temp_tnode_ptr->operand1 = Pop_tnode_ptr(Sem_stack,Sem_sp_ptr); 
	  trash = Pop_tnode_ptr(Sem_stack,Sem_sp_ptr);
	  Push_tnode_ptr(Sem_stack,Sem_sp_ptr,Temp_tnode_ptr);
	  break;
	  
	case   4  : // polynomial -> polynomial + term
	  Temp_tnode_ptr = Unexp_tnode_alloc(); 
	  Temp_tnode_ptr->_operator = ADDITION;
	  Temp_tnode_ptr->operand2 = Pop_tnode_ptr(Sem_stack,Sem_sp_ptr); 
	  trash = Pop_tnode_ptr(Sem_stack,Sem_sp_ptr);
	  Temp_tnode_ptr->operand1 = Pop_tnode_ptr(Sem_stack,Sem_sp_ptr); 
	  Push_tnode_ptr(Sem_stack,Sem_sp_ptr,Temp_tnode_ptr);
	  break;
	  
	case   5  : // polynomial -> polynomial - term
	  Temp_tnode_ptr = Unexp_tnode_alloc();
	  Temp_tnode_ptr->_operator = SUBTRACTION;
	  Temp_tnode_ptr->operand2 = Pop_tnode_ptr(Sem_stack,Sem_sp_ptr); 
	  trash = Pop_tnode_ptr(Sem_stack,Sem_sp_ptr);
	  Temp_tnode_ptr->operand1 = Pop_tnode_ptr(Sem_stack,Sem_sp_ptr); 
	  Push_tnode_ptr(Sem_stack,Sem_sp_ptr,Temp_tnode_ptr);
	  break;
	  
	case   6  : // term -> product
	  break;
      
	case   7  : // term -> int product
	  Temp_tnode_ptr = Unexp_tnode_alloc();
	  Temp_tnode_ptr->_operator = SCALAR_MULT;
	  Temp_tnode_ptr->operand2 = Pop_tnode_ptr(Sem_stack,Sem_sp_ptr); 
	  Temp_tnode_ptr->operand1 = Pop_tnode_ptr(Sem_stack,Sem_sp_ptr); 
	  Push_tnode_ptr(Sem_stack,Sem_sp_ptr,Temp_tnode_ptr);
	  break;
      
	case   8  : // product -> atom_or_dbl_atom
	  break;
	  
	case   9  : // product -> atom_or_dbl_atom ^ int
	  Temp_tnode_ptr = Unexp_tnode_alloc();
	  Temp_tnode_ptr->_operator = EXPONENTIATION;
	  Temp_tnode_ptr->operand2 = Pop_tnode_ptr(Sem_stack,Sem_sp_ptr); 
	  trash = Pop_tnode_ptr(Sem_stack,Sem_sp_ptr);
	  Temp_tnode_ptr->operand1 = Pop_tnode_ptr(Sem_stack,Sem_sp_ptr); 
	  Push_tnode_ptr(Sem_stack,Sem_sp_ptr,Temp_tnode_ptr);
	  break;
	  
	case   10 : // atom_or_dbl_atom -> atom_or_dbl_atom  * atom_or_dbl_atom
	  Temp_tnode_ptr = Unexp_tnode_alloc();
	  Temp_tnode_ptr->_operator = JORDAN_PROD;
	  Temp_tnode_ptr->operand2 = Pop_tnode_ptr(Sem_stack,Sem_sp_ptr); 
	  trash = Pop_tnode_ptr(Sem_stack,Sem_sp_ptr);
	  Temp_tnode_ptr->operand1 = Pop_tnode_ptr(Sem_stack,Sem_sp_ptr); 
	  Push_tnode_ptr(Sem_stack,Sem_sp_ptr,Temp_tnode_ptr);
	  break;
	  
	case   11 : // atom_or_dbl_atom -> ratom 
	  break;
	  
	case  12  : // atom_or_dbl_atom -> double_atom 
	  break;
	  
	case  13  : // double_atom -> atom ratom 
	  Temp_tnode_ptr = Unexp_tnode_alloc();
	  Temp_tnode_ptr->_operator = JUXT_PROD;
	  Temp_tnode_ptr->operand2 = Pop_tnode_ptr(Sem_stack,Sem_sp_ptr); 
	  Temp_tnode_ptr->operand1 = Pop_tnode_ptr(Sem_stack,Sem_sp_ptr); 
	  Push_tnode_ptr(Sem_stack,Sem_sp_ptr,Temp_tnode_ptr);
	  break;
	  
	case  14  : // ratom -> atom 
	case  15  : // atom -> small_letter 
	case  16  : // atom -> commutator 
	case  17  : // atom -> associator 
	case  18  : // atom -> jacobi 
	case  19  : // atom -> jordan_associator 
	case  20  : // atom -> operator_product 
	case  21  : // atom -> artificial_word 
	  break;
	
	case  22  : // atom -> ( polynomial- )
	  trash = Pop_tnode_ptr(Sem_stack,Sem_sp_ptr);
	  Temp_tnode_ptr =  Pop_tnode_ptr(Sem_stack,Sem_sp_ptr); 
	  trash = Pop_tnode_ptr(Sem_stack,Sem_sp_ptr);
	  Push_tnode_ptr(Sem_stack,Sem_sp_ptr,Temp_tnode_ptr);
	  break;
	  
	case  23  : // commutator -> [ polynomial-, polynomial- ] 
	  Temp_tnode_ptr = Unexp_tnode_alloc();
	  Temp_tnode_ptr->_operator = COMMUTATION;
	  trash = Pop_tnode_ptr(Sem_stack,Sem_sp_ptr);
	  Temp_tnode_ptr->operand2 = Pop_tnode_ptr(Sem_stack,Sem_sp_ptr); 
	  trash = Pop_tnode_ptr(Sem_stack,Sem_sp_ptr);
	  Temp_tnode_ptr->operand1 = Pop_tnode_ptr(Sem_stack,Sem_sp_ptr); 
	  trash = Pop_tnode_ptr(Sem_stack,Sem_sp_ptr);
	  Push_tnode_ptr(Sem_stack,Sem_sp_ptr,Temp_tnode_ptr);
	  break;
	  
	case  24  : // associator -> ( polynomial-, polynomial-, polynomial- ) 
	  Temp_tnode_ptr = Unexp_tnode_alloc();
	  Temp_tnode_ptr->_operator = ASSOCIATION;
	  trash = Pop_tnode_ptr(Sem_stack,Sem_sp_ptr);
	  Temp_tnode_ptr->operand3 = Pop_tnode_ptr(Sem_stack,Sem_sp_ptr); 
	  trash = Pop_tnode_ptr(Sem_stack,Sem_sp_ptr);
	  Temp_tnode_ptr->operand2 = Pop_tnode_ptr(Sem_stack,Sem_sp_ptr); 
	  trash = Pop_tnode_ptr(Sem_stack,Sem_sp_ptr);
	  Temp_tnode_ptr->operand1 = Pop_tnode_ptr(Sem_stack,Sem_sp_ptr); 
	  trash = Pop_tnode_ptr(Sem_stack,Sem_sp_ptr);
	  Push_tnode_ptr(Sem_stack,Sem_sp_ptr,Temp_tnode_ptr);
	  break;
	  
	case  25  : // jacobi -> J(polynomial-,polynomial-,polynomial-) 
	  Temp_tnode_ptr = Unexp_tnode_alloc();
	  Temp_tnode_ptr->_operator = JACOBI;
	  trash = Pop_tnode_ptr(Sem_stack,Sem_sp_ptr);
	  Temp_tnode_ptr->operand3 = Pop_tnode_ptr(Sem_stack,Sem_sp_ptr); 
	  trash = Pop_tnode_ptr(Sem_stack,Sem_sp_ptr);
	  Temp_tnode_ptr->operand2 = Pop_tnode_ptr(Sem_stack,Sem_sp_ptr); 
	  trash = Pop_tnode_ptr(Sem_stack,Sem_sp_ptr);
	  Temp_tnode_ptr->operand1 = Pop_tnode_ptr(Sem_stack,Sem_sp_ptr); 
	  trash = Pop_tnode_ptr(Sem_stack,Sem_sp_ptr);
	  trash = Pop_tnode_ptr(Sem_stack,Sem_sp_ptr);
	  Push_tnode_ptr(Sem_stack,Sem_sp_ptr,Temp_tnode_ptr);
	  break;
	  
	case  26  : // jordan_associator -> <polynomial-,polynomial-,polynomial-> 
	  Temp_tnode_ptr = Unexp_tnode_alloc();
	  Temp_tnode_ptr->_operator = JORDAN_ASSOC;
	  trash = Pop_tnode_ptr(Sem_stack,Sem_sp_ptr);
	  Temp_tnode_ptr->operand3 = Pop_tnode_ptr(Sem_stack,Sem_sp_ptr); 
	  trash = Pop_tnode_ptr(Sem_stack,Sem_sp_ptr);
	  Temp_tnode_ptr->operand2 = Pop_tnode_ptr(Sem_stack,Sem_sp_ptr); 
	  trash = Pop_tnode_ptr(Sem_stack,Sem_sp_ptr);
	  Temp_tnode_ptr->operand1 = Pop_tnode_ptr(Sem_stack,Sem_sp_ptr); 
	  trash = Pop_tnode_ptr(Sem_stack,Sem_sp_ptr);
	  Push_tnode_ptr(Sem_stack,Sem_sp_ptr,Temp_tnode_ptr);
	  break;
	  
	case  27  : // polynomial- -> polynomial 
	  break;
	  
	case  28  : // operator_product -> { atom operator_list- } 
	  Temp_tnode_ptr = Unexp_tnode_alloc();
	  Temp_tnode_ptr->_operator = OPERATOR_PROD;
	  trash = Pop_tnode_ptr(Sem_stack,Sem_sp_ptr);
	  Temp_tnode_ptr->operand2 = Pop_tnode_ptr(Sem_stack,Sem_sp_ptr); 
	  Temp_tnode_ptr->operand1 = Pop_tnode_ptr(Sem_stack,Sem_sp_ptr); 
	  trash = Pop_tnode_ptr(Sem_stack,Sem_sp_ptr);
	  Push_tnode_ptr(Sem_stack,Sem_sp_ptr,Temp_tnode_ptr);
	  break;
	  
	case  29  : // operator_list- -> operator_list 
	case  30  : // operator_list -> operator 
	  break;
	  
	case  31  : // operator_list -> operator_list operator 
	  temp_tnode_ptr1 = Pop_tnode_ptr(Sem_stack,Sem_sp_ptr); 
	  Temp_tnode_ptr = Pop_tnode_ptr(Sem_stack,Sem_sp_ptr); 
	  temp_tnode_ptr2 = Temp_tnode_ptr;
	  while (temp_tnode_ptr2->next != NULL)
		temp_tnode_ptr2 = temp_tnode_ptr2->next;
	  temp_tnode_ptr2->next = temp_tnode_ptr1;
	  Push_tnode_ptr(Sem_stack,Sem_sp_ptr,Temp_tnode_ptr);
	  break;

	case  32  : // operator -> atom ' 
	  Temp_tnode_ptr = Unexp_tnode_alloc();
	  Temp_tnode_ptr->_operator = LEFT_MULT;
	  trash = Pop_tnode_ptr(Sem_stack,Sem_sp_ptr);
	  Temp_tnode_ptr->operand1 = Pop_tnode_ptr(Sem_stack,Sem_sp_ptr); 
	  Push_tnode_ptr(Sem_stack,Sem_sp_ptr,Temp_tnode_ptr);
	  break;
	  
	case  33  : // operator -> atom ` 
	  Temp_tnode_ptr = Unexp_tnode_alloc();
	  Temp_tnode_ptr->_operator = RIGHT_MULT;
	  trash = Pop_tnode_ptr(Sem_stack,Sem_sp_ptr);
	  Temp_tnode_ptr->operand1 = Pop_tnode_ptr(Sem_stack,Sem_sp_ptr); 
	  Push_tnode_ptr(Sem_stack,Sem_sp_ptr,Temp_tnode_ptr);
	  break;
	  
	case  34  : // artificial_word -> artificial_word- } 
	  break;
	  
	case  35  : // artificial_word- -> artificial_word- : atom 
	  temp_tnode_ptr1 = Pop_tnode_ptr(Sem_stack,Sem_sp_ptr); 
	  trash = Pop_tnode_ptr(Sem_stack,Sem_sp_ptr);
	  Temp_tnode_ptr = Pop_tnode_ptr(Sem_stack,Sem_sp_ptr); 
	  temp_tnode_ptr2 = Temp_tnode_ptr->operand2;
	  while (temp_tnode_ptr2->next != NULL)
		temp_tnode_ptr2 = temp_tnode_ptr2->next;
	  temp_tnode_ptr2->next = temp_tnode_ptr1;
	  Push_tnode_ptr(Sem_stack,Sem_sp_ptr,Temp_tnode_ptr);
	  break;
	  
	case  36  : // artificial_word -> W { int : atom 
	  Temp_tnode_ptr = Unexp_tnode_alloc();
	  Temp_tnode_ptr->_operator = ARTIFICIAL_WORD;
	  Temp_tnode_ptr->operand2 = Pop_tnode_ptr(Sem_stack,Sem_sp_ptr); 
	  trash = Pop_tnode_ptr(Sem_stack,Sem_sp_ptr);
	  Temp_tnode_ptr->operand1 = Pop_tnode_ptr(Sem_stack,Sem_sp_ptr); 
	  trash = Pop_tnode_ptr(Sem_stack,Sem_sp_ptr);
	  trash = Pop_tnode_ptr(Sem_stack,Sem_sp_ptr);
	  Push_tnode_ptr(Sem_stack,Sem_sp_ptr,Temp_tnode_ptr);
	  break;
	  
	default  :
	  std::cout << "Error : Invalid type" << std::endl;
    }
} // end of Reduce_semantics()






// Get a new tnode. Change its fields to represent a letter or an integer.
// MODIFIES: Sem_stack -- Semantic stack, stack of pointers to tnodes.
// args:                                                   
//     Sem_sp -- Semantic stack pointer.                   
//     Token -- Token number.                              
//     Current_letter -- letter to be stored in the stack. 
//     Current_int -- number to be stored in the stack.         
void Store_semantics (struct unexp_tnode *Sem_stack[], int Sem_sp, int Token,
					  char Current_letter, int Current_int)
{
  if (Token == LETTER) 
	{
	  Sem_stack[Sem_sp] = Unexp_tnode_alloc(); 
	  Sem_stack[Sem_sp]->_operator = SMALL_LETTER;
	  Sem_stack[Sem_sp]->s_letter = Current_letter;
    }
  else if (Token == INT) 
	{
	  Sem_stack[Sem_sp] =  Unexp_tnode_alloc();
	  Sem_stack[Sem_sp]->_operator = SCALAR;
	  Sem_stack[Sem_sp]->scalar_num = Current_int;
    }
}




// MODIFIES:                                                      
//     Sem_stack -- Semantic stack, stack of pointers to tnodes.  
//     *Sem_sp_ptr -- Semantic stack pointer.                     
// arg: Temp_tnode_ptr -- Pointer to tnode which is to be pushed.
void Push_tnode_ptr (struct unexp_tnode *Sem_stack[], int* Sem_sp_ptr,
					 struct unexp_tnode* Temp_tnode_ptr)
{
  if (*Sem_sp_ptr < STACK_SIZE - 1)
	{
	  Sem_stack[++(*Sem_sp_ptr)] = Temp_tnode_ptr;
	}
  else
	{
	  std::cout << "semantic stack overflow" << std::endl;
	}
}




// MODIFIES:                                                      
//     Sem_stack -- Semantic stack, stack of pointers to tnodes.  
//     *Sem_sp_ptr -- Semantic stack pointer.                     
// RETURNS: Pointer to the tnode stored at the top of the stack.
struct unexp_tnode* Pop_tnode_ptr (struct unexp_tnode *Sem_stack[], 
								   int* Sem_sp_ptr)
{
  if ((*Sem_sp_ptr > 0) && (*Sem_sp_ptr < STACK_SIZE))
	{
	  return (Sem_stack[(*Sem_sp_ptr)--]);
	}
  else
	{
	  std::cout << "semantic stack underflow" << std::endl;
	  return (NULL);
	}
}



// REQUIRES:                                                      
//     Sem_stack -- Semantic stack, stack of pointers to tnodes.  
//     Sem_sp -- Semantic stack pointer.                           
// NOTE: Called only when DEBUG_PARSE flag is on.                    
void Print_Sem_stack (struct unexp_tnode *Sem_stack[], int Sem_sp)
{
  for (int i=0; i<=Sem_sp; i++)
	{
	  if (Sem_stack[i] != NULL) 
		{
		  std::cout << "Sem_stack[" << i << "] = " << Sem_stack[i]->s_letter 
					<< std::endl;
		  std::cout << "Sem_stack[" << i << "] = " << Sem_stack[i]->scalar_num 
					<< std::endl;
		}
	}
}



// end of Po_semantics.cc
