/*
  file:         SparseReduceMatrix.hh
  created:      Pasha Zusmanovich feb 21 2009
  modified:     pasha feb 18-19,mar 1,13 2016
  modification: - one return int -> bool
                - attempt to template Scalar
				- c++ file ext.
				- include once
*/

#ifndef __SPARSE_REDUCE_MATRIX_H
#define __SPARSE_REDUCE_MATRIX_H

#include "assert_not_null.hh"
#include "node_mgt.hh"



template<typename T> void Change_Element (Node<T>* Element_Ptr, T value)
{       
  // change the value in the node
  Element_Ptr->element = value;
}
                                        



template<typename T> 
T Get_Matrix_Element (Node<T>** Sparse_Matrix, int i, int j)
{
  // either return the element at location i,j or return a zero
  Node<T>* Tmp_Ptr = Sparse_Matrix[i];
  if (Tmp_Ptr != NULL)
    {
	  while ((Tmp_Ptr != NULL) && (Tmp_Ptr->column <= j))
        {
		  if (Tmp_Ptr->column == j)
            {       
			  return (Tmp_Ptr->element);
            }
		  Tmp_Ptr = Tmp_Ptr->Next_Node;
        }
	  return ((T)0);
    }
  else
    {
	  return ((T)0);
    }
} // end of Get_Matrix_Element()





template<typename T> void Insert_Node (Node<T>** Sparse_Matrix, int matrow, 
									   Node<T>* New_Node, Node<T>* Prev_Ptr)
{
  Node<T>* temp_node;

  // if it is to be the first element of the row
  if (Prev_Ptr == NULL)
    {
	  New_Node->Next_Node = Sparse_Matrix[matrow];
	  Sparse_Matrix[matrow] = New_Node;
    }
  // otherwise it has a predessor and possibly a successor
  else
    {
	  temp_node = Prev_Ptr->Next_Node;
	  Prev_Ptr->Next_Node = New_Node;
	  New_Node->Next_Node = temp_node;
    }
} // end of Insert_Node()





/* Locate_Node() either returns NULL or the node pointer to the node
   before the one we want. If it returns NULL then we know that 
   Either:
      1. Row empty or
      2. we are before the one we want and we are first or
   If it is non NULL then 
   Either:
      1. We are before the one we want and there is another node after us
      2. We are before the column we want and there are no nodes after us
*/
template<typename T> 
Node<T>* Locate_Node (Node<T>** Sparse_Matrix, int row, int col)
{
  Node<T>* Prev_Ptr = NULL;
  Node<T>* Curr_Ptr = Sparse_Matrix[row];

  // We will either be at the end of the linked list or at the node just before
  // the row,col that we want
  while ((Curr_Ptr != NULL) && (col > Curr_Ptr->column))
    {
	  Prev_Ptr = Curr_Ptr;
	  Curr_Ptr = Curr_Ptr->Next_Node;
    }
  return (Prev_Ptr);
} // end of Locate_Node()





template<typename T> bool Row_empty (const Node<T>** Sparse_Matrix, 
									 const unsigned int row)
{
  return (Sparse_Matrix[row] == NULL);
}



#endif // __SPARSE_REDUCE_MATRIX_H
// end of SparseReduceMatrix.hh
