/*
  file:    lidia_alg_num.hh
  purpose: wrapper around LiDIA alg_number class
  created: pasha jun 23,26-28 2019
*/

#ifndef __LIDIA_ALG_NUM_H
#define __LIDIA_ALG_NUM_H

#include <LiDIA/alg_number.h>
#include "albert.hh"

class alg_number: public Scalar<LiDIA::alg_number>
{
public:

  inline alg_number() {}

  // ctor
  inline alg_number (const long i)
  {
	// more efficient?
	this->val = LiDIA::alg_number ((LiDIA::bigint)i, Albert::NF);
  }

  friend inline alg_number operator- (const alg_number& X)
  {
	return (alg_number (-X.val));
  }

  friend inline alg_number operator+ (const alg_number& X, const alg_number& Y)
  {
	return (alg_number (X.val + Y.val));
  }

  friend inline alg_number operator* (const alg_number& X, const alg_number& Y)
  {
	return (alg_number (X.val * Y.val));
  }

  
  // 1/X
  friend inline alg_number S_inv (const alg_number& X) 
  {
	if (X.val.is_zero())
	  {
		std::cerr << "division by zero in LiDIA" << std::endl;
		exit (-1);
	  }
	return (alg_number (inverse (X.val)));
  }


  inline alg_number& operator*= (const alg_number& X)
  {
	*this = *this * X;
	return (*this);
  }

private:
  
  // ctor
  inline alg_number (const LiDIA::alg_number& a)
  {
	this->val = a;
  }

}; // end of class alg_number



#endif // __LIDIA_ALG_NUM_H
// end of lidia_alg_num.hh
