/*
  file:         scalar.hh
  purpose:      definition of Scalar class (base class for implementations of 
                elements of the ground field)
  created:      pasha mar 2,8,10 2016
  modified:     pasha jun 27-29 2019
  modification: - added more operators from the derived classes
                - S_inv() wrapper for mpq transferred to a separate file
*/

// Elements of the base field can be any class (whether derived from Scalar or
// not) which implements the following:
//    unary operator -
//    binary operator +
//    binary operator *
//    binary operator *=
//    unary operator S_inv (inversion)
//    binary operator ==
//    binary operator !=
//    operator << (printing to ostream)
//    conversion from 0 and 1


#ifndef __SCALAR_H
#define __SCALAR_H


template<typename T> class Scalar
{
public:

  friend inline bool operator== (const Scalar<T>& X, const Scalar<T>& Y)
  {
	return (X.val == Y.val);
  }
  
  friend inline bool operator!= (const Scalar<T>& X, const Scalar<T>& Y)
  {
	return (X.val != Y.val);
  }

  // templates should be in headers
  friend std::ostream& operator<< (std::ostream& o, const Scalar<T>& X)
  {
	return (o << X.val);
  }
  
protected:
  T val;

}; // end of class Scalar



#endif // __SCALAR_H
// end of scalar.hh
